@@ -31,6 +31,8 @@
 #include <gtk/gtk.h>
 #include <glib-object.h>
 #include <libosso.h>
+#include <cairo.h>
+#include <values.h>
 
 #include <libintl.h>
 #include <libgnomevfs/gnome-vfs.h>
@@ -42,7 +44,9 @@
 
 #include <X11/extensions/Xrender.h>
 #include <X11/extensions/Xfixes.h>
+#include <X11/Xatom.h>
 
+#define WORKAREA_ATOM "_NET_WORKAREA"
 #define HOME_DIR g_get_home_dir()
 #define IMAGEFILE "/usr/share/pixmaps/plgrip.png"
 
@@ -50,11 +54,55 @@
 HD_DEFINE_PLUGIN(PersonalLauncher, personal_launcher, HILDON_DESKTOP_TYPE_HOME_ITEM)
 
 struct _PersonalLauncherPrivate {
-	GtkWidget	*desktop;
+	GtkWidget *desktop;
+	gdouble alphaComp,redComp,blueComp,greenComp;
+	gboolean animation;
+	gint radius;
+	gint trans;
+	gboolean layoutCheck;
+	gint layoutStyle;
+	gint iconSize;
 };
 
 static void create_ui(PersonalLauncher *plugin);
 
+static void get_workarea(GtkAllocation *allocation)
+{
+	unsigned long n;
+	unsigned long extra;
+	int format;
+	int status;
+	Atom property = XInternAtom (GDK_DISPLAY(),WORKAREA_ATOM,FALSE);
+	Atom realType;
+
+	union
+	{
+		unsigned char *char_value;
+		int *int_value;
+	} value;
+
+	status=XGetWindowProperty(GDK_DISPLAY(),GDK_ROOT_WINDOW(),property,0L,4L,0,XA_CARDINAL,&realType,&format,&n,&extra,(unsigned char**) &value.char_value);
+
+	if (status==Success && realType==XA_CARDINAL && format==32 && n==4 &&value.char_value!=NULL)
+	{
+		allocation->x=value.int_value[0];
+		allocation->y=value.int_value[1];
+		allocation->width=value.int_value[2];
+		allocation->height=value.int_value[3]-60;
+	}
+	else
+	{
+		allocation->x=0;
+		allocation->y=0;
+		allocation->width=0;
+		allocation->height=0;
+	}
+	if (value.char_value)
+	{
+		XFree(value.char_value);
+	}
+}
+
 static gboolean launcher_changed(PersonalLauncher *plugin)
 {
 	plugin->updateTimeout=0;
@@ -97,7 +145,10 @@ static gint applet_expose(GtkWidget *widget, GdkEventExpose *event)
 	Picture		grip,mask;
 	gint		width,height;
 	XserverRegion	region;
-
+	cairo_t *cr;
+	gdouble o=0;
+	gint radius;
+	
 	if (GTK_WIDGET_DRAWABLE(widget) == FALSE) {
 		return FALSE;
 	}
@@ -110,7 +161,8 @@ static gint applet_expose(GtkWidget *widget, GdkEventExpose *event)
 		return FALSE;
 	}
 	
-	hildon_desktop_picture_and_mask_from_file(IMAGEFILE,
+	if (priv->layoutCheck)
+		hildon_desktop_picture_and_mask_from_file(IMAGEFILE,
 			&grip,
 			&mask,
 			&width,
@@ -121,37 +173,77 @@ static gint applet_expose(GtkWidget *widget, GdkEventExpose *event)
 		if (!plugin->updateTimeout)
 			plugin->updateTimeout=g_timeout_add(1000,(GSourceFunc)launcher_changed,plugin);
 	}
+	
 	plugin->clip.x = event->area.x - x_offset;
 	plugin->clip.y = event->area.y - y_offset;
-	plugin->clip.width = event->area.width;
-	plugin->clip.height = event->area.height;
+	plugin->clip.width = event->area.width + x_offset;
+	plugin->clip.height = event->area.height + y_offset;
 
 	region = XFixesCreateRegion(GDK_DISPLAY(), &plugin->clip, 1);
 
 	XFixesSetPictureClipRegion(GDK_DISPLAY(), picture, 0, 0, region);
 
-	color.red = color.blue = color.green = 0;
-	color.alpha = 0;
+	if (priv->redComp<0||priv->redComp>1||!priv->redComp)
+		priv->redComp=0;
+	if (priv->blueComp<0||priv->blueComp>1||!priv->blueComp)
+		priv->blueComp=0;
+	if (priv->greenComp<0||priv->greenComp>1||!priv->greenComp)
+		priv->greenComp=0;
+	if (priv->alphaComp<0||priv->alphaComp>1||!priv->alphaComp)
+		priv->alphaComp=0;
+	if (priv->radius<0||priv->radius>26||!priv->radius)
+		priv->radius=0;
+
+	color.red=color.blue=color.green=0;
+	color.alpha=0;
 
 	XRenderFillRectangle(GDK_DISPLAY(), PictOpSrc, picture, &color,
 			0, 0,
 			widget->allocation.width,
 			widget->allocation.height);
 
-	if (grip) {
-		XRenderComposite(GDK_DISPLAY(), PictOpSrc,
-				grip,
-				mask,
-				picture,
-				0, 0,
-				0, 0,
-				widget->allocation.width-width, widget->allocation.height-height,
-				widget->allocation.width, widget->allocation.height);
-	}
+	if (priv->layoutCheck)
+		if (grip) {
+			XRenderComposite(GDK_DISPLAY(), PictOpSrc,
+					grip,
+					mask,
+					picture,
+					0, 0,
+					0, 0,
+					widget->allocation.width-width, widget->allocation.height-height,
+					widget->allocation.width, widget->allocation.height);
+		}
 
 	XFixesDestroyRegion(GDK_DISPLAY(), region);
 	XRenderFreePicture(GDK_DISPLAY(), picture);
 
+	if ((priv->iconSize!=event->area.width)&&(priv->iconSize!=event->area.height))
+	{
+		cr=gdk_cairo_create(drawable);
+		cairo_set_source_rgba(cr,priv->redComp,priv->blueComp,priv->greenComp,priv->alphaComp);
+		cairo_move_to(cr,plugin->clip.x+priv->radius+o,plugin->clip.y+o);
+		cairo_line_to(cr,plugin->clip.x+plugin->clip.width-priv->radius,plugin->clip.y+o);
+		cairo_curve_to(cr,plugin->clip.x+plugin->clip.width-priv->radius-o,plugin->clip.y+o,plugin->clip.x+plugin->clip.width,plugin->clip.y,plugin->clip.x+plugin->clip.width-o,plugin->clip.y+priv->radius+o);
+		cairo_line_to(cr,plugin->clip.x+plugin->clip.width-o,plugin->clip.y+plugin->clip.height-priv->radius-o);
+		cairo_curve_to(cr,plugin->clip.x+plugin->clip.width-o,plugin->clip.y+plugin->clip.height-o-priv->radius,plugin->clip.x+plugin->clip.width,plugin->clip.y+plugin->clip.height,plugin->clip.x+plugin->clip.width-o-priv->radius,plugin->clip.y+plugin->clip.height-o);
+		cairo_line_to(cr,plugin->clip.x+priv->radius+o,plugin->clip.y+plugin->clip.height-o);
+		cairo_curve_to(cr,plugin->clip.x+o+priv->radius,plugin->clip.y+plugin->clip.height-o,plugin->clip.x,plugin->clip.y+plugin->clip.height,plugin->clip.x+o,plugin->clip.y+plugin->clip.height-o-priv->radius);
+		cairo_line_to(cr,plugin->clip.x+o,plugin->clip.y+o+priv->radius);
+		cairo_curve_to(cr,plugin->clip.x+o,plugin->clip.y+o+priv->radius,plugin->clip.x,plugin->clip.y,plugin->clip.x+o+priv->radius,plugin->clip.y+o);
+		cairo_fill(cr);
+	}
+	else
+	{
+		cr=gdk_cairo_create(drawable);
+		cairo_set_source_rgba(cr,priv->redComp,priv->blueComp,priv->greenComp,priv->alphaComp);
+		cairo_move_to(cr,plugin->clip.x,plugin->clip.y);
+		cairo_line_to(cr,plugin->clip.x+plugin->clip.width,plugin->clip.y);
+		cairo_line_to(cr,plugin->clip.x+plugin->clip.width,plugin->clip.y+plugin->clip.height);
+		cairo_line_to(cr,plugin->clip.x,plugin->clip.y+plugin->clip.height);
+		cairo_line_to(cr,plugin->clip.x,plugin->clip.y);
+		cairo_fill(cr);
+	}
+	
 	if (plugin->queueRefresh)
 	{
 		create_ui(plugin);
@@ -167,29 +259,59 @@ static void applet_realize(GtkWidget *widget)
 	GTK_WIDGET_CLASS(personal_launcher_parent_class)->realize(widget);
 }
 
-static void activation_animation(GtkWidget *widget, GdkEventButton *event, gint trans)
+static void activation_animation(GtkWidget *widget, GdkEventButton *event, PersonalLauncherPrivate *priv)
 {
-	if (trans>=0)
+	if (priv->animation)
 	{
-		gint width,height;
-
 		GdkPixbuf *orig=gtk_image_get_pixbuf(gtk_bin_get_child(GTK_BIN(widget)));
-		width=gdk_pixbuf_get_width(orig);
-		height=gdk_pixbuf_get_height(orig);
+		gint width=gdk_pixbuf_get_width(orig);
+		gint height=gdk_pixbuf_get_height(orig);
 		GdkPixbufSimpleAnim *anim=gdk_pixbuf_simple_anim_new(width,height,30);
-
+		gint red=priv->redComp*255;
+		gint green=priv->greenComp*255;
+		gint blue=priv->blueComp*255;
+		gint alpha=priv->alphaComp*255;
+		gchar *colourString=NULL;
+		gint colour=0;
 		gint i;
-		for (i=1;i<width/4;i+=2)
+		gint curTrans=priv->trans;
+
+		/*if (g_strcasecmp(gtk_widget_get_name(widget),"first"))
+		{*/
+			colourString=g_strdup_printf("%02X%02X%02X%02X\n",red,green,blue,alpha);
+			colour=g_ascii_strtoll(colourString,NULL,16);
+			g_free(colourString);
+		/*}*/
+
+		for (i=0;i<23;i++)
 		{
-			GdkPixbuf *icon=gdk_pixbuf_scale_simple(orig,width-(i*2),height-(i*2),GDK_INTERP_NEAREST);
+			GdkPixbuf *icon=gdk_pixbuf_scale_simple(orig,width,height,GDK_INTERP_BILINEAR);
 			GdkPixbuf *good=gdk_pixbuf_new(GDK_COLORSPACE_RGB,TRUE,8,width,height);
-			gdk_pixbuf_fill(good,0x00000000);
-			gdk_pixbuf_composite(icon,good,0,0,width,height,0,0,1,1,GDK_INTERP_NEAREST,trans);
+			
+			good=gdk_pixbuf_add_alpha(good,FALSE,0,0,0);
+			if (i<12)
+			{
+				gdk_pixbuf_composite(icon,good,0,0,width,height,0,0,1,1,GDK_INTERP_BILINEAR,curTrans-10);
+				curTrans-=10;
+			}
+			else
+			{
+				gdk_pixbuf_composite(icon,good,0,0,width,height,0,0,1,1,GDK_INTERP_BILINEAR,curTrans+10);
+				curTrans+=10;
+			}
 			gdk_pixbuf_simple_anim_add_frame(anim,good);
 			gdk_pixbuf_unref(icon);
 			gdk_pixbuf_unref(good);
 		}
-		gdk_pixbuf_simple_anim_add_frame(anim,orig);
+		GdkPixbuf *icon=gdk_pixbuf_scale_simple(orig,width,height,GDK_INTERP_BILINEAR);
+		GdkPixbuf *good=gdk_pixbuf_new(GDK_COLORSPACE_RGB,TRUE,8,width,height);
+		
+		good=gdk_pixbuf_add_alpha(good,FALSE,0,0,0);
+		gdk_pixbuf_composite(icon,good,0,0,width,height,0,0,1,1,GDK_INTERP_BILINEAR,priv->trans);
+		gdk_pixbuf_simple_anim_add_frame(anim,good);
+		gdk_pixbuf_unref(icon);
+		gdk_pixbuf_unref(good);
+
 		gtk_container_foreach(GTK_CONTAINER(widget),gtk_widget_destroy,widget);
 		GtkWidget *image=gtk_image_new_from_animation(GDK_PIXBUF_ANIMATION(anim));
 		gtk_container_add(GTK_CONTAINER(widget),image);
@@ -215,7 +337,7 @@ static void execute_command(GtkWidget *item, GdkEventButton *event, gint index)
 	GKeyFile* keyFile;
 	GError *error=NULL;
 	gboolean result;
-
+	
 	if (index>=0)
 	{
 		snprintf(entry,5,"%d",index);
@@ -232,19 +354,27 @@ static void execute_command(GtkWidget *item, GdkEventButton *event, gint index)
 			service=g_key_file_get_string(keyFile,entry,"service",NULL);
 		}
 		g_key_file_free(keyFile);
-
+		
 		if (rootFlag)
 		{
-			executable=g_strconcat("sudo ",executable,NULL);
+			FILE *fp=NULL;
+			fp=fopen("/usr/bin/rootsh","rb");
+			if (fp!=NULL)
+			{
+				fclose(fp);
+				executable=g_strconcat("rootsh ",executable,NULL);
+			}
+			else
+				executable=g_strconcat("sudo ",executable,NULL);
 		}
-
+		
 		if (termFlag)
 		{
 			osso_context_t *osso=osso_initialize("personal-menu-launch","0.5",TRUE,NULL);
 			osso_return_t retVal;
-		
+			
 			executable = g_strconcat("sh -c \"", g_strescape(executable, NULL), " ; read x\"", NULL);
-
+			
 			retVal=osso_rpc_async_run(osso,"com.nokia.xterm","/com/nokia/xterm","com.nokia.xterm","run_command",NULL,NULL,DBUS_TYPE_STRING,executable,DBUS_TYPE_INVALID);
 			if (retVal==OSSO_OK)
 			{
@@ -287,7 +417,18 @@ static void execute_command(GtkWidget *item, GdkEventButton *event, gint index)
 	}
 	else
 	{
-		run_setup(NULL);
+		osso_context_t *osso;
+		
+		osso=osso_initialize("personal-menu","0.9",TRUE,NULL);
+		osso_rpc_async_run(osso,
+			"com.nokia.personallaunchersetup",
+			"/com/nokia/personallaunchersetup",
+			"com.nokia.personallaunchersetup",
+			"top_application", NULL,NULL,
+			DBUS_TYPE_STRING,"settings",
+			DBUS_TYPE_INVALID);
+			
+			osso_deinitialize(osso);
 	}
 	g_free(appName);
 	g_free(iconName);
@@ -297,16 +438,13 @@ static void execute_command(GtkWidget *item, GdkEventButton *event, gint index)
 
 static void create_ui(PersonalLauncher *plugin)
 {
+	PersonalLauncherPrivate *priv = PERSONAL_LAUNCHER_GET_PRIVATE(plugin);
 	GtkIconTheme *icon_theme;
 	GtkWidget *image;
-	GtkWidget *event;
 	gchar* filename;
     gboolean fileExists;
     GKeyFile* keyFile;
-    gboolean animation;
-	int iconSize=64;
-	int trans=255;
-
+		
 	if (GTK_IS_FIXED(plugin->layout))
 	{
 		gtk_container_foreach(GTK_CONTAINER(plugin->layout), (GtkCallback)gtk_widget_destroy, NULL);
@@ -314,7 +452,9 @@ static void create_ui(PersonalLauncher *plugin)
 	else
 	{
     	plugin->layout=gtk_fixed_new();
+    	gtk_widget_set_name(plugin->layout,"layout");
 	}
+
 	keyFile=g_key_file_new();
     filename=g_build_filename(HOME_DIR,"/.personal_launcher.rc",NULL);
     fileExists=g_key_file_load_from_file(keyFile,filename,G_KEY_FILE_KEEP_COMMENTS,NULL);
@@ -326,34 +466,33 @@ static void create_ui(PersonalLauncher *plugin)
 		int padding=1;
 		int minRows=1;
 		int numCols=1;
-		int entryPerColumn;
-		gboolean layoutBool;
 		gsize numEntry;
 		GError *error=NULL;
 		gchar **retVal=g_key_file_get_groups(keyFile,&numEntry);
 		char entry[5];
 		gboolean found;
+		GtkWidget *event;
 		g_strfreev(retVal);
 
 		numEntry--;
-		iconSize=g_key_file_get_integer(keyFile,"config","icon size",&error);
+		priv->iconSize=g_key_file_get_integer(keyFile,"config","icon size",&error);
 		if (error)
 		{
-			iconSize=40;
+			priv->iconSize=40;
 			g_error_free(error);
 			error=NULL;
 		}
-		trans=g_key_file_get_integer(keyFile,"config","transparency",&error);
+		priv->trans=g_key_file_get_integer(keyFile,"config","transparency",&error);
 		if (error)
 		{
-			trans=255;
+			priv->trans=255;
 			g_error_free(error);
 			error=NULL;
 		}
-		animation=g_key_file_get_boolean(keyFile,"config","animation",&error);
+		priv->animation=g_key_file_get_boolean(keyFile,"config","animation",&error);
 		if (error)
 		{
-			animation=TRUE;
+			priv->animation=TRUE;
 			g_error_free(error);
 			error=NULL;
 		}
@@ -364,22 +503,57 @@ static void create_ui(PersonalLauncher *plugin)
 			g_error_free(error);
 			error=NULL;
 		}
-		layoutBool=g_key_file_get_boolean(keyFile,"config","auto layout",&error);
+		priv->layoutCheck=g_key_file_get_boolean(keyFile,"config","auto layout",&error);
 		if (error)
 		{
-			layoutBool=TRUE;
+			priv->layoutCheck=TRUE;
 			g_error_free(error);
 			error=NULL;
 		}
-		if (!layoutBool)
+		priv->layoutStyle=g_key_file_get_integer(keyFile,"config","layout style",&error);
+		if (error)
+		{
+			priv->layoutStyle=1;
+			g_error_free(error);
+			error=NULL;
+		}
+		priv->alphaComp=g_key_file_get_double(keyFile,"config","background transparency",&error);
+		if (error)
+		{
+			priv->alphaComp=0;
+			g_error_free(error);
+			error=NULL;
+		}
+		priv->redComp=g_key_file_get_double(keyFile,"config","background red",&error);
+		if (error)
+		{
+			priv->redComp=0;
+			g_error_free(error);
+			error=NULL;
+		}
+		priv->blueComp=g_key_file_get_double(keyFile,"config","background blue",&error);
+		if (error)
+		{
+			priv->blueComp=0;
+			g_error_free(error);
+			error=NULL;
+		}
+		priv->greenComp=g_key_file_get_double(keyFile,"config","background green",&error);
+		if (error)
+		{
+			priv->greenComp=0;
+			g_error_free(error);
+			error=NULL;
+		}
+		priv->radius=g_key_file_get_integer(keyFile,"config","background radius",&error);
+		if (error)
+		{
+			priv->radius=0;
+			g_error_free(error);
+			error=NULL;
+		}
+		if (!priv->layoutCheck)
 		{
-			minRows=g_key_file_get_integer(keyFile,"config","num rows",&error);
-			if (error)
-			{
-				minRows=1;
-				g_error_free(error);
-				error=NULL;
-			}
 			numCols=g_key_file_get_integer(keyFile,"config","num cols",&error);
 			if (error)
 			{
@@ -387,13 +561,28 @@ static void create_ui(PersonalLauncher *plugin)
 				g_error_free(error);
 				error=NULL;
 			}
+	
+			if (priv->layoutStyle!=2)
+			{
+				GtkAllocation workarea={0,0,800,480};
+				
+				get_workarea(&workarea);
+				numCols=workarea.width/(priv->iconSize+padding);
+				minRows=workarea.height/(priv->iconSize+padding);
+			}			
+			hildon_desktop_home_item_set_resize_type(
+				HILDON_DESKTOP_HOME_ITEM(plugin),
+				HILDON_DESKTOP_HOME_ITEM_RESIZE_NONE);
 		}
 		else
 		{
-			minRows=plugin->clip.height/(iconSize+padding);
-			numCols=plugin->clip.width/(iconSize+padding);
+			minRows=plugin->clip.height/(priv->iconSize+padding);
+			numCols=plugin->clip.width/(priv->iconSize+padding);
+			
+			hildon_desktop_home_item_set_resize_type(
+				HILDON_DESKTOP_HOME_ITEM(plugin),
+				HILDON_DESKTOP_HOME_ITEM_RESIZE_BOTH);
 		}
-		entryPerColumn=minRows;
 
 		snprintf(entry,5,"%d",i);
 		found=g_key_file_has_group(keyFile,entry);
@@ -402,6 +591,8 @@ static void create_ui(PersonalLauncher *plugin)
 			gchar* appName;
 			gchar* iconName;
 			gchar* executable=NULL;
+			gint xOffset=0;
+			gint yOffset=0;
 
 			sprintf(entry,"%d",i);
 			appName=g_key_file_get_string(keyFile,entry,"app name",NULL);
@@ -412,44 +603,64 @@ static void create_ui(PersonalLauncher *plugin)
 			{
 				event=gtk_event_box_new();
 				gtk_event_box_set_visible_window(GTK_EVENT_BOX(event),FALSE);
-				if (iconSize!=0)
+				if (priv->iconSize!=0)
 				{
+					gint width,height;
 					icon_theme = gtk_icon_theme_get_default ();
-					GdkPixbuf *icon=gtk_icon_theme_load_icon(icon_theme,iconName,iconSize,0,NULL);
+					GdkPixbuf *icon=gtk_icon_theme_load_icon(icon_theme,iconName,priv->iconSize,0,NULL);
 					if (icon==NULL)
 					{
-						icon=gtk_icon_theme_load_icon(icon_theme,"qgn_list_gene_default_app",iconSize,0,NULL);
+						icon=gtk_icon_theme_load_icon(icon_theme,"qgn_list_gene_default_app",priv->iconSize,0,NULL);
 					}
-					GdkPixbuf *scale=gdk_pixbuf_scale_simple(icon,iconSize,iconSize,GDK_INTERP_NEAREST);
-					GdkPixbuf *comp=gdk_pixbuf_new(GDK_COLORSPACE_RGB,TRUE,8,iconSize,iconSize);
-					gdk_pixbuf_fill(comp,0x00000000);
-					gdk_pixbuf_composite(scale,comp,0,0,iconSize,iconSize,0,0,1,1,GDK_INTERP_NEAREST,trans);
-					image=gtk_image_new_from_pixbuf(comp);
+					width=gdk_pixbuf_get_width(icon);
+					height=gdk_pixbuf_get_height(icon);
+					if (width>height)
+					{
+						gint scaleRatio=((gdouble)height/(gdouble)width)*priv->iconSize;
+						icon=gdk_pixbuf_scale_simple(icon,priv->iconSize,scaleRatio,GDK_INTERP_BILINEAR);
+						yOffset=(gdouble)(priv->iconSize-scaleRatio)/2;
+					}
+					else if (height>width)
+					{
+						gint scaleRatio=((gdouble)width/(gdouble)height)*priv->iconSize;
+						icon=gdk_pixbuf_scale_simple(icon,scaleRatio,priv->iconSize,GDK_INTERP_BILINEAR);
+						xOffset=(gdouble)(priv->iconSize-scaleRatio)/2;
+					}
+					else
+					{
+						icon=gdk_pixbuf_scale_simple(icon,priv->iconSize,priv->iconSize,GDK_INTERP_BILINEAR);
+					}
+					gdk_pixbuf_add_alpha(icon,FALSE,0,0,0);
+					image=gtk_image_new_from_pixbuf(icon);
 
 					gdk_pixbuf_unref(icon);
-					gdk_pixbuf_unref(scale);
-					gdk_pixbuf_unref(comp);
+					
 					gtk_container_add(GTK_CONTAINER(event),image);
 				}
-				if (animation)
-				{
-					g_signal_connect(G_OBJECT(event),"button-press-event",G_CALLBACK(activation_animation),trans);
-				}
-				else
-				{
-					g_signal_connect(G_OBJECT(event),"button-press-event",G_CALLBACK(activation_animation),-1);
-				}
+				g_signal_connect(G_OBJECT(event),"button-press-event",G_CALLBACK(activation_animation),priv);
 				g_signal_connect(G_OBJECT(event),"button-release-event",G_CALLBACK(execute_command),i);
 			}
-
-			gtk_fixed_put(GTK_FIXED(plugin->layout),event,x*(iconSize+padding),y*(iconSize+padding));
-			y++;
-			if (y>entryPerColumn-1)
+			
+			gtk_fixed_put(GTK_FIXED(plugin->layout),event,x*(priv->iconSize+padding)+xOffset,y*(priv->iconSize+padding)+yOffset);
+			
+			if ((priv->layoutStyle==0)&&(!priv->layoutCheck))
 			{
-				y=0;
+				y++;
+				if (y>minRows-1)
+				{
+					y=0;
+					x++;
+				}
+			}
+			else
+			{
 				x++;
+				if (x>numCols-1)
+				{
+					x=0;
+					y++;
+				}
 			}
-
 			snprintf(entry,5,"%d",++i);
 			found=g_key_file_has_group(keyFile,entry);
 			g_free(appName);
@@ -459,7 +670,7 @@ static void create_ui(PersonalLauncher *plugin)
 	}
 	else
 	{
-		event=gtk_event_box_new();
+		GtkWidget *event=gtk_event_box_new();
 		gtk_event_box_set_visible_window(GTK_EVENT_BOX(event),FALSE);
 		icon_theme = gtk_icon_theme_get_default ();
 		GdkPixbuf *icon=gtk_icon_theme_load_icon(icon_theme,"personal-launcher",64,0,NULL);
@@ -473,8 +684,6 @@ static void create_ui(PersonalLauncher *plugin)
 		g_object_unref(G_OBJECT(icon));
 		gtk_container_add(GTK_CONTAINER(event),image);
 
-		g_signal_connect(G_OBJECT(event),"button-press-event",G_CALLBACK(activation_animation),255);
-
 		g_signal_connect(G_OBJECT(event),"button-release-event",G_CALLBACK(execute_command),-1);
 
 		gtk_fixed_put(GTK_FIXED(plugin->layout),event,0,0);
@@ -486,20 +695,34 @@ static void create_ui(PersonalLauncher *plugin)
     if (gtk_bin_get_child(GTK_BIN(plugin)))
     	gtk_widget_show_all(plugin->layout);
     else
+    {
  		gtk_container_add(GTK_CONTAINER(plugin),plugin->layout);
- 		
- 	gtk_widget_show_all(GTK_WIDGET(plugin));
+ 		gtk_widget_show_all(GTK_WIDGET(plugin));
+	}
+	if (!priv->layoutCheck)
+		gtk_widget_set_size_request(plugin,-1,-1);
 }
 
 static void
 settings_popup(PersonalLauncher *plugin)
 {
 	PersonalLauncherPrivate *priv = PERSONAL_LAUNCHER_GET_PRIVATE(plugin);
+	osso_context_t *osso;
+	osso_rpc_t retval;
 	
-	run_setup(priv->desktop->window);
+	osso=osso_initialize("personal-launcher","0.6",TRUE,NULL);
+	osso_rpc_async_run(osso,
+		"com.nokia.personallaunchersetup",
+		"/com/nokia/personallaunchersetup",
+		"com.nokia.personallaunchersetup",
+		"top_application", NULL,NULL,
+		DBUS_TYPE_STRING,"settings",
+		DBUS_TYPE_INVALID);
+		
+	osso_deinitialize(osso);
 }
 
-GtkWidget *settings_menu_entry(HildonDesktopHomeItem *applet, GtkWidget *parent)
+GtkWidget *settings_menu_entry(HildonDesktopHomeItem *applet, GtkWindow *parent)
 {
 	PersonalLauncher *plugin = PERSONAL_LAUNCHER(applet);
 	PersonalLauncherPrivate *priv = PERSONAL_LAUNCHER_GET_PRIVATE(PERSONAL_LAUNCHER(applet));
@@ -507,7 +730,7 @@ GtkWidget *settings_menu_entry(HildonDesktopHomeItem *applet, GtkWidget *parent)
 
 	priv->desktop = parent;
 	item = gtk_menu_item_new_with_label("Personal Launcher");
-
+	
 	g_signal_connect_swapped(G_OBJECT(item), "activate",
 			G_CALLBACK(settings_popup), plugin);
 
